open Softcore open Lexing

let non_regular_string = "Error: the input schema is not regular."

(* The main function, called with the input channel *)
let main ~verbose ?fname ch =
  if ch = stdin then print_endline "Taking input from stdin.";
  let lexbuf = 
    let lexbuf' = from_channel ch in
    match fname with None -> lexbuf' | Some fn -> { lexbuf' with lex_curr_p = { lexbuf'.lex_curr_p with pos_fname = fn } } in
  print_endline 
  (try
    let (sch,cstr) = Parser.main Lexer.token lexbuf in
    (if verbose then (Schema.to_string sch) ^ "\n" else "")
    ^
    (if not (Schema.is_regular sch) then non_regular_string
    else (if Satisfiability.unsatisfiable (Regstab.regstab ~cstr sch) then "unsatisfiable" else "satisfiable"))
  with Failure s -> s
  );
  print_newline ();
  flush stdout

(* Handling of command-line options, calls main *)
let _ =
  let channel = ref None in
  let verbose = ref false in
  let options = [
    "-verbose", Arg.Set(verbose), "Be verbose (i.e. currently, print the input schema after parsing)"
  ] in
  let usage = "regstab [file]    - says if the schema in file is satisfiable, schema taken on stdin if no file provided" in
  let file s = channel := Some(s) in
  Arg.parse options file usage;
  match !channel with
  |None      -> main stdin ~verbose:(!verbose)
  |Some file -> read_wrap ~f:(main ~verbose:(!verbose) ~fname:file) file

