open Softcore

let non_regular_string = "Error: the input schema is not regular."

(* The main function, called with the input channel and options *)
let main ch ~dimacs ~cnf ~param =
  if ch = stdin then print_endline "Taking input from stdin.";
  let lexbuf  = Lexing.from_channel ch in
  (try
    let (sch,_) = Parser.main Lexer.token lexbuf in
    if Schema.is_regular sch then (
      let f = Schema.compute_schema sch param in
      if dimacs 
      then Formula.print_dimacs f
      else (if cnf
        then Formula.print_cnf f
        else Formula.print f
      )
    )
    else print_endline non_regular_string
  with Failure s -> print_endline s);
  print_newline ();
  flush stdout

(* Handling of command-line options, calls main *)
let _ =
  let param  = ref None
  and file   = ref None
  and dimacs = ref true
  and cnf    = ref true in
  let options = Arg.align [
    "-cnf"  , Arg.Set(cnf)      , " Display in conjunctive normal form (useful only when -H is set)";
    "-D"    , Arg.Set  (dimacs),  " Display in DIMACS format (default)";
    "-H"    , Arg.Clear(dimacs),  " Display in a human readable format instead of DIMACS";
  ]
  and usage = "sch2cnf n [file]  - computes the instance of a schema for value n of the parameter\n"
  and call_main s = 
      match !param with
      |None ->
          (try param := Some(int_of_string s)
          with _ -> match !file with None -> file := Some(s) |_ -> failwith "Too many arguments.")
      |Some(_) ->
          (match !file with
          |None    -> file := Some(s)
          |Some(_) -> failwith "Too many arguments.")
          in
  Arg.parse options call_main usage;
  match !param,!file with
    |None,_          -> failwith "At least the value of the parameter shall be provided on the command-line!"
    |Some(p),None    -> main ~dimacs:(!dimacs) ~cnf:(!cnf) ~param:p stdin
    |Some(p),Some(f) -> read_wrap ~f:(main ~dimacs:(!dimacs) ~cnf:(!cnf) ~param:p) f
